def _get_documenting_directories():
   '''Get top-level Abjad directories to search for
   classes and functions to document in public API.
   '''

   return ['components', 'interfaces', 'marks', 'spanners', 'tools', ]
